@@ -5,6 +5,7 @@ from pulp.constants import PulpError
 from pulp.apis import *
 from pulp import LpVariable, LpProblem, lpSum, LpConstraintVar, LpFractionConstraint
 from pulp import constants as const
+from pulp.utilities import makeDict
 import unittest
 
 
@@ -45,6 +46,15 @@ class PuLPTest(unittest.TestCase):
         unittest.TestCase.__init__(self, testName)
         self.solver = solver.copy()
 
+    def tearDown(self):
+        for ext in ['mst', 'log', 'lp', 'mps', 'sol']:
+            filename = "{}.{}".format(self._testMethodName, ext)
+            try:
+                os.remove(filename)
+            except:
+                pass
+        pass
+
     def test_pulp_001(self):
         """
         Test that a variable is deleted when it is suptracted to 0
@@ -297,6 +307,43 @@ class PuLPTest(unittest.TestCase):
         pulpTestCheck(prob, self.solver, [const.LpStatusOptimal], {x: 3, y: -0.5, z: 7},
                       objective=64.95)
 
+    def test_pulp_022(self):
+        # Initial value
+        prob = LpProblem("test022", const.LpMinimize)
+        x = LpVariable("x", 0, 4)
+        y = LpVariable("y", -1, 1)
+        z = LpVariable("z", 0, None, const.LpInteger)
+        prob += x + 4 * y + 9 * z, "obj"
+        prob += x + y <= 5, "c1"
+        prob += x + z >= 10, "c2"
+        prob += -y + z == 7.5, "c3"
+        x.setInitialValue(3)
+        y.setInitialValue(-0.5)
+        z.setInitialValue(7)
+        if self.solver.name in ['GUROBI', 'GUROBI_CMD', 'CPLEX_CMD', 'CPLEX_PY']:
+            warnings.warn("CBC gives a wrong solution with warmStart.")
+            self.solver.optionsDict['warmStart'] = True
+        print("\t Testing MIP solution")
+        pulpTestCheck(prob, self.solver, [const.LpStatusOptimal], {x: 3, y: -0.5, z: 7})
+
+    def test_pulp_023(self):
+        # Initial value (fixed)
+        prob = LpProblem("test023", const.LpMinimize)
+        x = LpVariable("x", 0, 4)
+        y = LpVariable("y", -1, 1)
+        z = LpVariable("z", 0, None, const.LpInteger)
+        prob += x + 4 * y + 9 * z, "obj"
+        prob += x + y <= 5, "c1"
+        prob += x + z >= 10, "c2"
+        prob += -y + z == 7.5, "c3"
+        solution = {x: 4, y: -0.5, z: 7}
+        for v in [x, y, z]:
+            v.setInitialValue(solution[v])
+            v.fixValue()
+        self.solver.optionsDict['warmStart'] = True
+        print("\t Testing MIP solution")
+        pulpTestCheck(prob, self.solver, [const.LpStatusOptimal], solution)
+
     def test_pulp_030(self):
         # relaxed MIP
         prob = LpProblem("test030", const.LpMinimize)
@@ -369,6 +416,26 @@ class PuLPTest(unittest.TestCase):
         else:
             pulpTestCheck(prob, self.solver, [const.LpStatusInfeasible])
 
+    def test_pulp_061(self):
+        # Integer Infeasible
+        prob = LpProblem('sample', const.LpMaximize)
+
+        dummy = LpVariable('dummy')
+        c1 = LpVariable('c1', 0, 1, const.LpBinary)
+        c2 = LpVariable('c2', 0, 1, const.LpBinary)
+
+        prob += dummy
+        prob += c1 + c2 == 2
+        prob += c1 <= 0
+        print("\t Testing another integer infeasible problem")
+        if self.solver.__class__ in [GUROBI_CMD]:
+            pulpTestCheck(prob, self.solver, [const.LpStatusNotSolved])
+        elif self.solver.__class__ in [GLPK_CMD]:
+            # GLPK_CMD returns InfeasibleOrUnbounded
+            pulpTestCheck(prob, self.solver, [const.LpStatusInfeasible, const.LpStatusUndefined])
+        else:
+            pulpTestCheck(prob, self.solver, [const.LpStatusInfeasible])
+
     def test_pulp_070(self):
         # Column Based modelling of test_pulp_1
         prob = LpProblem("test070", const.LpMinimize)
@@ -587,6 +654,208 @@ class PuLPTest(unittest.TestCase):
         else:
             pulpTestCheck(prob, self.solver, [const.LpStatusUnbounded])
 
+    def test_pulpTestAll(self):
+        """
+        Test the availability of the function pulpTestAll
+        """
+        print("\t Testing the availability of the function pulpTestAll")
+        from pulp import pulpTestAll
+
+    def test_export_dict_LP(self):
+        prob = LpProblem("test_export_dict_LP", const.LpMinimize)
+        x = LpVariable("x", 0, 4)
+        y = LpVariable("y", -1, 1)
+        z = LpVariable("z", 0)
+        w = LpVariable("w", 0)
+        prob += x + 4 * y + 9 * z, "obj"
+        prob += x + y <= 5, "c1"
+        prob += x + z >= 10, "c2"
+        prob += -y + z == 7, "c3"
+        prob += w >= 0, "c4"
+        data = prob.to_dict()
+        var1, prob1 = LpProblem.from_dict(data)
+        x, y, z, w = [var1[name] for name in ['x', 'y', 'z', 'w']]
+        print("\t Testing continuous LP solution")
+        pulpTestCheck(prob1, self.solver, [const.LpStatusOptimal], {x: 4, y: -1, z: 6, w: 0})
+
+    def test_export_json_LP(self):
+        name = self._testMethodName
+        prob = LpProblem(name, const.LpMinimize)
+        x = LpVariable("x", 0, 4)
+        y = LpVariable("y", -1, 1)
+        z = LpVariable("z", 0)
+        w = LpVariable("w", 0)
+        prob += x + 4 * y + 9 * z, "obj"
+        prob += x + y <= 5, "c1"
+        prob += x + z >= 10, "c2"
+        prob += -y + z == 7, "c3"
+        prob += w >= 0, "c4"
+        filename = name + '.json'
+        prob.to_json(filename, indent=4)
+        var1, prob1 = LpProblem.from_json(filename)
+        try:
+            os.remove(filename)
+        except:
+            pass
+        x, y, z, w = [var1[name] for name in ['x', 'y', 'z', 'w']]
+        print("\t Testing continuous LP solution")
+        pulpTestCheck(prob1, self.solver, [const.LpStatusOptimal], {x: 4, y: -1, z: 6, w: 0})
+
+    def test_export_dict_MIP(self):
+        import copy
+        prob = LpProblem("test_export_dict_MIP", const.LpMinimize)
+        x = LpVariable("x", 0, 4)
+        y = LpVariable("y", -1, 1)
+        z = LpVariable("z", 0, None, const.LpInteger)
+        prob += x + 4 * y + 9 * z, "obj"
+        prob += x + y <= 5, "c1"
+        prob += x + z >= 10, "c2"
+        prob += -y + z == 7.5, "c3"
+        data = prob.to_dict()
+        data_backup = copy.deepcopy(data)
+        var1, prob1 = LpProblem.from_dict(data)
+        x, y, z = [var1[name] for name in ['x', 'y', 'z']]
+        print("\t Testing MIP solution")
+        pulpTestCheck(prob1, self.solver, [const.LpStatusOptimal], {x: 3, y: -0.5, z: 7})
+        # we also test that we have not modified the dictionary when importing it
+        self.assertDictEqual(data, data_backup)
+
+    def test_export_dict_max(self):
+        prob = LpProblem("test_export_dict_max", const.LpMaximize)
+        x = LpVariable("x", 0, 4)
+        y = LpVariable("y", -1, 1)
+        z = LpVariable("z", 0)
+        w = LpVariable("w", 0)
+        prob += x + 4 * y + 9 * z, "obj"
+        prob += x + y <= 5, "c1"
+        prob += x + z >= 10, "c2"
+        prob += -y + z == 7, "c3"
+        prob += w >= 0, "c4"
+        data = prob.to_dict()
+        var1, prob1 = LpProblem.from_dict(data)
+        x, y, z, w = [var1[name] for name in ['x', 'y', 'z', 'w']]
+        print("\t Testing maximize continuous LP solution")
+        pulpTestCheck(prob1, self.solver, [const.LpStatusOptimal], {x: 4, y: 1, z: 8, w: 0})
+
+    def test_export_solver_dict_LP(self):
+        prob = LpProblem("test_export_dict_LP", const.LpMinimize)
+        x = LpVariable("x", 0, 4)
+        y = LpVariable("y", -1, 1)
+        z = LpVariable("z", 0)
+        w = LpVariable("w", 0)
+        prob += x + 4 * y + 9 * z, "obj"
+        prob += x + y <= 5, "c1"
+        prob += x + z >= 10, "c2"
+        prob += -y + z == 7, "c3"
+        prob += w >= 0, "c4"
+        data = self.solver.to_dict()
+        solver1 = get_solver_from_dict(data)
+        print("\t Testing continuous LP solution")
+        pulpTestCheck(prob, solver1, [const.LpStatusOptimal], {x: 4, y: -1, z: 6, w: 0})
+
+    def test_export_solver_json(self):
+        name = self._testMethodName
+        prob = LpProblem(name, const.LpMinimize)
+        x = LpVariable("x", 0, 4)
+        y = LpVariable("y", -1, 1)
+        z = LpVariable("z", 0)
+        w = LpVariable("w", 0)
+        prob += x + 4 * y + 9 * z, "obj"
+        prob += x + y <= 5, "c1"
+        prob += x + z >= 10, "c2"
+        prob += -y + z == 7, "c3"
+        prob += w >= 0, "c4"
+        self.solver.mip = True
+        logFilename = name + '.log'
+        if self.solver.name == 'CPLEX_CMD':
+            self.solver.optionsDict = dict(gapRel=0.1, gapAbs=1, maxMemory=1000,
+                                           maxNodes=1, threads=1, logPath=logFilename,
+                                           warmStart=True)
+        elif self.solver.name in ['GUROBI_CMD', 'COIN_CMD', 'PULP_CBC_CMD']:
+            self.solver.optionsDict = dict(gapRel=0.1, gapAbs=1, threads=1, logPath=logFilename,
+                                           warmStart=True)
+        filename = name + '.json'
+        self.solver.to_json(filename, indent=4)
+        solver1 = get_solver_from_json(filename)
+        try:
+            os.remove(filename)
+        except:
+            pass
+        print("\t Testing continuous LP solution")
+        pulpTestCheck(prob, solver1, [const.LpStatusOptimal], {x: 4, y: -1, z: 6, w: 0})
+
+    def test_timeLimit(self):
+        name = self._testMethodName
+        prob = LpProblem(name, const.LpMinimize)
+        x = LpVariable("x", 0, 4)
+        y = LpVariable("y", -1, 1)
+        z = LpVariable("z", 0)
+        w = LpVariable("w", 0)
+        prob += x + 4 * y + 9 * z, "obj"
+        prob += x + y <= 5, "c1"
+        prob += x + z >= 10, "c2"
+        prob += -y + z == 7, "c3"
+        prob += w >= 0, "c4"
+        self.solver.timeLimit = 20
+        # CHOCO has issues when given a time limit
+        if self.solver.name != 'PULP_CHOCO_CMD':
+            pulpTestCheck(prob, self.solver, [const.LpStatusOptimal], {x: 4, y: -1, z: 6, w: 0})
+    
+    def test_assignInvalidStatus(self):
+        print("\t Testing invalid status")
+        t = LpProblem('test')
+        Invalid = -100
+        self.assertRaises(const.PulpError, lambda: t.assignStatus(Invalid))
+        self.assertRaises(const.PulpError, lambda: t.assignStatus(0, Invalid))
+
+    def test_logPath(self):
+        name = self._testMethodName
+        prob = LpProblem(name, const.LpMinimize)
+        x = LpVariable("x", 0, 4)
+        y = LpVariable("y", -1, 1)
+        z = LpVariable("z", 0)
+        w = LpVariable("w", 0)
+        prob += x + 4 * y + 9 * z, "obj"
+        prob += x + y <= 5, "c1"
+        prob += x + z >= 10, "c2"
+        prob += -y + z == 7, "c3"
+        prob += w >= 0, "c4"
+        logFilename = name + '.log'
+        self.solver.optionsDict['logPath'] = logFilename
+        if self.solver.name in ['CPLEX_PY', 'CPLEX_CMD', 'GUROBI', 'GUROBI_CMD', 'PULP_CBC_CMD', 'COIN_CMD']:
+            pulpTestCheck(prob, self.solver, [const.LpStatusOptimal], {x: 4, y: -1, z: 6, w: 0})
+            if not os.path.exists(logFilename):
+                raise PulpError("Test failed for solver: {}".format(self.solver))
+            if not os.path.getsize(logFilename):
+                raise PulpError("Test failed for solver: {}".format(self.solver))
+
+    def test_makeDict_behavior(self):
+        """
+        Test if makeDict is returning the expected value.
+        """
+        headers = [["A", "B"], ["C", "D"]]
+        values = [[1, 2], [3, 4]]
+        target = {"A": {"C": 1, "D": 2}, "B": {"C": 3, "D": 4}}
+        dict_with_default = makeDict(headers, values, default=0)
+        dict_without_default = makeDict(headers, values)
+        print("\t Testing makeDict general behavior")
+        self.assertEqual(dict_with_default, target)
+        self.assertEqual(dict_without_default, target)
+
+    def test_makeDict_default_value(self):
+        """
+        Test if makeDict is returning a default value when specified.
+        """
+        headers = [["A", "B"], ["C", "D"]]
+        values = [[1, 2], [3, 4]]
+        dict_with_default = makeDict(headers, values, default=0)
+        dict_without_default = makeDict(headers, values)
+        print("\t Testing makeDict default value behavior")
+        # Check if a default value is passed
+        self.assertEqual(dict_with_default["X"]["Y"], 0)
+        # Check if a KeyError is raised
+        _func = lambda: dict_without_default["X"]["Y"]
+        self.assertRaises(KeyError, _func)
 
 def pulpTestCheck(prob, solver, okstatus, sol=None,
                   reducedcosts=None,
@@ -600,43 +869,41 @@ def pulpTestCheck(prob, solver, okstatus, sol=None,
         status = prob.solve(solver, **kwargs)
     if status not in okstatus:
         dumpTestProblem(prob)
-        print("Failure: status ==", status, "not in", okstatus)
-        print("Failure: status ==", const.LpStatus[status], "not in", \
-              [const.LpStatus[s] for s in okstatus])
-        raise PulpError("Tests failed for solver %s" % solver)
+        raise PulpError("Tests failed for solver {}:\nstatus == {} not in {}\nstatus == {} not in {}".
+                        format(solver, status, okstatus,
+                               const.LpStatus[status], [const.LpStatus[s] for s in okstatus]))
     if sol is not None:
         for v, x in sol.items():
             if abs(v.varValue - x) > eps:
                 dumpTestProblem(prob)
-                print("Test failed: var", v, "==", v.varValue, "!=", x)
-                raise PulpError("Tests failed for solver %s" % solver)
+                raise PulpError("Tests failed for solver {}:\nvar {} == {} != {}".
+                                format(solver, v, v.varValue, x))
     if reducedcosts:
         for v, dj in reducedcosts.items():
             if abs(v.dj - dj) > eps:
                 dumpTestProblem(prob)
-                print("Test failed: var.dj", v, "==", v.dj, "!=", dj)
-                raise PulpError("Tests failed for solver %s" % solver)
+                raise PulpError("Tests failed for solver {}:\nTest failed: var.dj {} == {} != {}".
+                                format(solver, v, v.dj, dj))
     if duals:
         for cname, p in duals.items():
             c = prob.constraints[cname]
             if abs(c.pi - p) > eps:
                 dumpTestProblem(prob)
-                print("Test failed: constraint.pi", cname, "==", c.pi, "!=", p)
-                raise PulpError("Tests failed for solver %s" % solver)
+                raise PulpError("Tests failed for solver {}:\nconstraint.pi {} == {} != {}".
+                                format(solver, cname, c.pi, p))
     if slacks:
         for cname, slack in slacks.items():
             c = prob.constraints[cname]
             if abs(c.slack - slack) > eps:
                 dumpTestProblem(prob)
-                print("Test failed: constraint.slack", cname, "==",
-                      c.slack, "!=", slack)
-                raise PulpError("Tests failed for solver %s" % solver)
+                raise PulpError("Tests failed for solver {}:\nconstraint.slack {} == {} != {}".
+                                format(solver, cname, c.slack, slack))
     if objective is not None:
         z = prob.objective.value()
         if abs(z - objective) > eps:
             dumpTestProblem(prob)
-            print("Test failed: objective ", z, " != ", objective)
-            raise PulpError("Tests failed for solver %s" % solver)
+            raise PulpError("Tests failed for solver {}:\nobjective {} != {}".
+                            format(solver, z, objective))
 
 
 def suite():
@@ -653,6 +920,7 @@ def suite():
                PYGLPK,
                YAPOSIB,
                PULP_CHOCO_CMD,
+               CHOCO_CMD,
                MIPCL_CMD,
                MOSEK
                ]
